# `html-base64.pl`

An HTML filter for encoding images in `<img src="...">`  and `<a href="...">`
as Base64 data URLS.

## Synopsis

``````html
<p><img src="lalune.jpg" alt="La Lune JPEG" /> </p>
<p><img src="lalune.png" data-mime="image/png" alt="La Lune PNG" /> </p>
<p><a href="lalune.jpg" class="base64">La Lune</a></p>
<p><a href="ignored/path/to/img.jpg" data-base64="lalune.jpg">La Lune
ditto</a></p>
``````

``````sh
perl html-base64.pl orig/doc.html >base64/doc.html
``````

``````html
<p><img alt="La Lune JPEG" src="data:image/jpeg;_9j_4AAQSkZJRgABAQEAeAB4AAD_2w..."></p>
<p><img alt="La Lune PNG" src="data:image/png;iVBORw0KGgoAAAANSUhEUgAAAPoAAA..."></p>
<p><a href="data:image/jpeg;_9j_4AAQSkZJRgABAQEAeAB4AAD_2w...">La Lune</a></p>
<p><a href="data:image/jpeg;_9j_4AAQSkZJRgABAQEAeAB4AAD_2w...">La Lune
ditto</a></p>
``````

## Installation

1.  You may need to install perl:

    (If you are on Windows install Strawberry Perl!)

    <https://www.perl.org/get.html>
    
2.  Install the Perl modules dependencies:

    ``````sh
    cpan App::cpanminus
    cpanm  Mojo::DOM URI::Escape MIME::Base64 Path::Tiny
    ``````
    
    **NOTE**: [Mojo::DOM][] is part of [Mojolicious][] which is a rather hefty
    dependency. However I choose to use it anyway since in my experience it
    offers the best way to edit HTML while preserving the validity of the
    HTML and your sanity!
    
[Mojo::DOM]: https://metacpan.org/pod/Mojo::DOM
[Mojolicious]: https://metacpan.org/pod/Mojolicious

## Description

This script will 

-   assume that input/output is UTF-8 encoded,
-   read in HTML source from stdin,
-   replace image source urls pointing at local image files with
    data URLs containing the base-64 encoded image data, and
-   write the modified HTML to stdout.

It will also convert anchor links pointing at images provided these
links have a class `base64` or a custom attribute `data-base64` (see below!)

The script will assume that the MIME type of the image file is
`image/jpeg`. If this is not correct you must provide a custom
attribute `data-mime` with the correct MIME type as value, e.g.
`data-mime="image/png"`.

If an image or link element has a custom attribute `data-base64` the
script will assume that the value of that attribute is the path to a
local image file and fetch the image data from that file; in this
case the original `src` or `href` attribute value will be ignored.

Elements whose `src` or `href` attribute value already starts
with `data:` will be skipped unless the element also has a `data-base64`
attribute as described above.

If the `data-base64`, `src` or `href` attribute of an image or link
element doesn't point at a local image file the script will print a
warning to stderr and skip that element.

The class `base64` and the custom attributes `data-base64` and
`data-mime` will be removed from an element upon successful
conversion.

If you got this script from where you should there are some bundled
example files, including a Makefile for converting a directory of
Markdown files to HTML with [Pandoc][], piping them through this script
and writing the result to a directory of HTML files.

## Author

Benct Philip Jonsson <bpjonsson@gmail.com>

## Copyright and license

This software is copyright (c) 2022 by Benct Philip Jonsson.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

http://dev.perl.org/licenses/

